/* MEX function to solve SDP 

*/
// Last Modified: A. d'Aspremont, Feb 2009
// http://www.carva.org/alexandre.daspremont


#include "ColDSPCA.h" 

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	// For main loop
	double *Amat;
	double *Xmat,*Xavmat,*grad,*ovals,*movals,*gaps,Bnd,ps,stepscal,*cput,targetg;
	double *bufmata,*probv,*cprobv,*bufccmat,*bufcovmat,alpha,beta,stepsz,bufnrm;
	double *rndn;
	int n,i,j,k,n2,cc,idum=-1,*sidx;
	int info,arpinfo,Nperiod,maxiter;
	double cputime;int left_h=0,left_m=0,left_s=0;
	clock_t start_time=(double)clock();
	double *evec,eigv,swsgn;
	int nummatvec,nconv,arcount,ncv,nev,incx=1;
	char which[2]="LM";double tol=1e-6; int maxitrarp=1000; // ARPACK parameters
		
	// Inputs
	Amat=mxGetPr(prhs[0]);	
	n = (int)mxGetN(prhs[0]);
	Bnd = mxGetScalar(prhs[1]); // Bound on ratings
	ps = mxGetScalar(prhs[2]);
	stepscal = mxGetScalar(prhs[3]);
	maxiter =(int)mxGetScalar(prhs[4]);
	info =(int)mxGetScalar(prhs[5]);
	targetg =mxGetScalar(prhs[6]);
	cc=(int)(ps*n);n2=n*n;
	// Outputs 
	plhs[0] = mxCreateDoubleMatrix(n,n, mxREAL);
	Xmat = mxGetPr(plhs[0]);
	plhs[1] = mxCreateDoubleMatrix(n,n, mxREAL);
	Xavmat = mxGetPr(plhs[1]);
	plhs[2] = mxCreateDoubleMatrix(maxiter+1, 1, mxREAL);
	ovals = mxGetPr(plhs[2]);
	plhs[3] = mxCreateDoubleMatrix(maxiter+1, 1, mxREAL);
	movals = mxGetPr(plhs[3]);
	plhs[4] = mxCreateDoubleMatrix(maxiter+1, 1, mxREAL);
	gaps = mxGetPr(plhs[4]);
	plhs[5] = mxCreateDoubleMatrix(maxiter+1, 1, mxREAL);
	cput = mxGetPr(plhs[5]);

	// Allocate
	evec=(double*)calloc(n,sizeof(double));
	probv=(double*)calloc(n,sizeof(double));
	cprobv=(double*)calloc(n+1,sizeof(double));
	bufmata=(double*)calloc(n,sizeof(double));
	grad=(double*)calloc(n*n,sizeof(double));	
	bufccmat=(double*)calloc(n*cc,sizeof(double));
	bufcovmat=(double*)calloc(cc*cc,sizeof(double));
	rndn=(double*)calloc(cc,sizeof(double));
	sidx=(int*)calloc(cc,sizeof(int));
	
	// Initialize X matrix
	cblas_dcopy(n2,Amat,incx,Xmat,incx);	
	// Start main loop
	stepsz=stepscal/sqrt(maxiter);Nperiod=maxiter/info;
	for(k=0;k<=maxiter;k++){
		// Compute sampling probability
		bufnrm=doubnorm2(Xmat,n2);bufnrm=bufnrm*bufnrm;
		for(i=0;i<n;i++){probv[i]=doubnorm2(Xmat+i*n,n);probv[i]=probv[i]*probv[i]/bufnrm;}
		cprobv[0]=0.0;cprobv[1]=probv[0];for(i=1;i<n;i++){cprobv[i+1]=cprobv[i]+probv[i];}
		// Sample from it
		for(i=0;i<cc;i++){rndn[i]=ran1(&idum);}
		for(i=0;i<cc;i++){sidx[i]=n-1;j=n-2;while(rndn[i]<cprobv[j+1]){sidx[i]=j;j--;}}
		// Create subsampled matrix
		for(i=0;i<cc;i++){
			cblas_dcopy(n,Xmat+sidx[i]*n,incx,bufccmat+i*n,incx);
			alpha=1.0/sqrt(cc*probv[sidx[i]]);cblas_dscal(n,alpha,bufccmat+i*n,incx);}
		// Compute eig decomposition
		nev=1;ncv=iminf(3,cc); // TODO: Should get a better size estimate for ncv here (cf. arpack manual)
		arpinfo=simlrarpack(bufccmat,n,cc,nev,ncv,tol,which,maxitrarp,0,&eigv,bufmata,&nconv,&nummatvec,&arcount);
		// Reconstruct eigenvectors
		if (eigv>=tol){alpha=1.0/sqrt(eigv);beta=0.0;}else{alpha=0.0;beta=0.0;}
		cblas_dgemv(CblasColMajor,CblasNoTrans,n,cc,alpha,bufccmat,n,bufmata,incx,beta,evec,incx);
		// Get gradient and step
		// Estimate sign of eigenvalues
		alpha=1.0;beta=0.0;
		cblas_dgemv(CblasColMajor,CblasNoTrans,n,n,alpha,Xmat,n,evec,incx,beta,probv,incx);
		swsgn=dsignf(doubdot(probv,evec,n));
		if (eigv<=tol){swsgn=0.0;}
		// Form gradient matrix
		cblas_dcopy(n,evec,incx,bufmata,incx);
		alpha=swsgn;cblas_dscal(n,alpha,bufmata,incx);
		alpha=1.0;beta=0.0;
		cblas_dgemm(CblasColMajor,CblasNoTrans,CblasTrans,n,n,1,alpha,evec,n,bufmata,n,beta,grad,n);
		// Take step
		alpha=-stepsz;cblas_daxpy(n2,alpha,grad,incx,Xmat,incx);
		// Project
		for(i=0;i<n2;i++){
			if (Xmat[i]<Amat[i]-Bnd) Xmat[i]=Amat[i]-Bnd;
			if (Xmat[i]>Amat[i]+Bnd) Xmat[i]=Amat[i]+Bnd;}
		// Update stats and average	
		ovals[k]=0.0;
		ovals[k]+=sqrt(dabsf(eigv));
		alpha=k;cblas_dscal(n2,alpha,Xavmat,incx);
		alpha=1.0;cblas_daxpy(n2,alpha,Xmat,incx,Xavmat,incx);
		alpha=1.0/(k+1.0);cblas_dscal(n2,alpha,Xavmat,incx);
		// Report progress periodically 
		cputime=(double)((clock()-start_time)/CLOCKS_PER_SEC);
		cput[k]=cputime;
		if (k%Nperiod==0){
			// Compute gap and objective
			// Compute eig decomposition to form dual point
			nev=1;ncv=iminf(3,n); // TODO: Should get a better size estimate for ncv here (cf. arapck manual)
			arpinfo=simlrarpack(Xavmat,n,n,nev,ncv,tol,which,maxitrarp,0,&eigv,evec,&nconv,&nummatvec,&arcount);
			movals[k]+=sqrt(dabsf(eigv));
			// Get sign of eigenvalues
			alpha=1.0;beta=0.0;
			cblas_dgemv(CblasColMajor,CblasNoTrans,n,n,alpha,Xavmat,n,evec,incx,beta,probv,incx);
			swsgn=dsignf(doubdot(probv,evec,n));			
			// Form gradient (dual) matrix
			cblas_dcopy(n,evec,incx,bufmata,incx);
			alpha=swsgn;cblas_dscal(n,alpha,bufmata,incx);
			alpha=1.0;beta=0.0;
			cblas_dgemm(CblasColMajor,CblasNoTrans,CblasTrans,n,n,1,alpha,evec,n,bufmata,n,beta,grad,n);						
			gaps[k]=movals[k]; // Record gap
			gaps[k]-=doubdot(grad,Amat,n2);
			gaps[k]+=Bnd*doubasum(grad,n2);
			left_h=(int)floor(cputime/3600.0);left_m=(int)floor(cputime/60.0-left_h*60);left_s=(int)floor(cputime-left_h*3600-left_m*60);
			mexPrintf("Iter: %.3e   Obj: %.4e    Gap: %.4e   CPU Time: %2dh %2dm %2ds\n",(double)(k),movals[k],gaps[k],left_h,left_m,left_s);
			mexEvalString("drawnow;");
			//if (gaps[k]/gaps[0]<=targetg){break;}
			if (movals[k]<=targetg){break;}
		}}
	// Free everything
	free(evec);free(probv);free(cprobv);free(bufmata);
	free(grad);free(bufccmat);free(bufcovmat);free(sidx);free(rndn);
}