/* MEX function to call ARPACK directly
[vec,vals,nconv]=SpMaxEig(A,nev,verb,tol,maxitr)

Inputs:
 A:			A symmetric matrix (either sparse or dense)
 nev:		Number of eigs requested
 verb:		Vervose if 1, no output if 0
 tol:		Numerical tolerance in ARPACK
 maxitr:	Max Number of iterations in arpack

Outputs:
 vec:		Eigenvectors
 vals:		Eiegnvalues
 nconv:		Number of eigs which converged in ARPACK

*/
// Last Modified: A. d'Aspremont, April 2008
// http://www.carva.org/alexandre.daspremont


#include "LmEigArp.h" 

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	double *eig,*evec,*Amat,*eigconv;
	int n,verb;
	int info,nummatvec,nconv,arcount,ncv;
	char which[2]="LA";
	double tol;
	int nev,maxitr;
	
	// Inputs
	Amat=mxGetPr(prhs[0]);
	n = (int)mxGetN(prhs[0]);
	if (n==1) {n=(int)sqrt(mxGetM(prhs[0]));}
	nev =(int)mxGetScalar(prhs[1]);
	verb =(int)mxGetScalar(prhs[2]);
	tol=mxGetScalar(prhs[3]);
	maxitr=(int)mxGetScalar(prhs[4]); 
	// Outputs
	plhs[0] = mxCreateDoubleMatrix(n, nev, mxREAL);
	evec = mxGetPr(plhs[0]);
	plhs[1] = mxCreateDoubleMatrix(nev, 1, mxREAL);
	eig = mxGetPr(plhs[1]);
	plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
	eigconv = mxGetPr(plhs[2]);


	// Allocate
	ncv=4; // TODO: Should get a better size estimate here (cf. arapck manual)
	info=simsqarpack(Amat,n,nev,ncv,tol,which,maxitr,verb,eig,evec,&nconv,&nummatvec,&arcount);
	*eig=sqrt(*eig);
	eigconv[0]=nconv;
	if ((info!=0)&&(verb==1)) {mexPrintf("SpMaxEig: Not all eigs converged in Arpack. iter=%d.\n",arcount);} 
}

// To reactivate breakpoints in XCODE. Clean all expressions and breakpoints. Start debugging, *then* place 
// a new breakpoint before starting the code
// SDK creates issues in Release version