/* MEX function to solve SDP 

*/
// Last Modified: A. d'Aspremont, Feb 2009
// http://www.carva.org/alexandre.daspremont


#include "SampSpectralNorm.h" 

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	// For main loop
	double *Amat,ps;
	double *bufveca,*probv,*cprobv,*bufccmat,*bufcovmat,alpha,beta,bufnrm;
	double *rndn;
	int n,i,j,n2,cc,idum=-1,*sidx;
	int info,maxiter;
	// For LAPACK
	int work_size; 
	int lwork,inflapack,indmax;
	double *workvec;
	char jobz[1]="V",uplo[1]="U";
	// For Arpack & BLAS
	double *evec,*eigout,eigv=0;
	int nummatvec,nconv,arcount,ncv,nev,incx=1;
	char which[2]="LM";double tol=1e-6; int maxitrarp=maxiter;
		
	// Inputs
	Amat=mxGetPr(prhs[0]);
	n = (int)mxGetN(prhs[0]);
	ps = mxGetScalar(prhs[1]);
	maxiter =(int)mxGetScalar(prhs[2]);
	tol=mxGetScalar(prhs[3]);
	cc=(int)(ps*n);n2=n*n;
	work_size=3*n+n*n;lwork=work_size; // For LAPACK
	// Outputs
	plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
	eigout = mxGetPr(plhs[0]);
	plhs[1] = mxCreateDoubleMatrix(n, 1, mxREAL);
	evec = mxGetPr(plhs[1]);

	// Allocate
	probv=(double*)calloc(n,sizeof(double));
	cprobv=(double*)calloc(n+1,sizeof(double));
	bufveca=(double*)calloc(n,sizeof(double));
	bufccmat=(double*)calloc(n*cc,sizeof(double));
	bufcovmat=(double*)calloc(cc*cc,sizeof(double));
	rndn=(double*)calloc(cc,sizeof(double));
	sidx=(int*)calloc(cc,sizeof(int));
	#ifndef usearpack
	workvec=(double *) calloc(work_size,sizeof(double)); // For LAPACK
	#endif
	
	// Compute sampling probability
	bufnrm=doubnorm2(Amat,n2);bufnrm=bufnrm*bufnrm;
	for(i=0;i<n;i++){probv[i]=doubnorm2(Amat+i*n,n);probv[i]=probv[i]*probv[i]/bufnrm;}
	cprobv[0]=0.0;cprobv[1]=probv[0];for(i=1;i<n;i++){cprobv[i+1]=cprobv[i]+probv[i];}
	// Sample from it
	for (i=0;i<cc;i++){rndn[i]=ran1(&idum);}
	for(i=0;i<cc;i++){sidx[i]=n-1;j=n-2;while(rndn[i]<cprobv[j+1]){sidx[i]=j;j--;}}
	// Create subsampled matrix
	for(i=0;i<cc;i++){
		cblas_dcopy(n,Amat+sidx[i]*n,incx,bufccmat+i*n,incx);
	alpha=1.0/sqrt(cc*probv[sidx[i]]);cblas_dscal(n,alpha,bufccmat+i*n,incx);}
	// Compute eig decomposition
	#ifdef usearpack // Switch between ARPACK and LAPACK
	nev=1;ncv=4; // TODO: Should get a better size estimate here (cf. arapck manual)
	info=simlrarpack(bufccmat,n,cc,nev,ncv,tol,which,maxitrarp,0,&eigv,bufveca,&nconv,&nummatvec,&arcount);
	#else
	// Square it 
	alpha=1.0;beta=0.0;
	cblas_dgemm(CblasColMajor,CblasTrans,CblasNoTrans,cc,cc,n,alpha,bufccmat,n,bufccmat,n,beta,bufcovmat,cc);	
	dsyev(jobz,uplo,&cc,bufcovmat,&cc,bufveca,workvec,&lwork,&inflapack);
	indmax=idxabsmax(bufveca,cc);eigv=bufveca[indmax];cblas_dcopy(cc,bufcovmat+indmax*cc,incx,bufveca,incx);
	#endif
	// Reconstruct eigenvector
	alpha=1.0/sqrt(eigv);beta=0.0;
	cblas_dgemv(CblasColMajor,CblasNoTrans,n,cc,alpha,bufccmat,n,bufveca,incx,beta,evec,incx);
	*eigout=sqrt(eigv);
	
	// Free everything
	free(bufveca);free(probv);free(cprobv);free(bufccmat);free(bufcovmat);
	free(sidx);free(rndn);
	#ifndef usearpack
	free(workvec);
	#endif
}