// MEX wrapper for the sparse_rank_one function in sparsesvd.c
// function [X,U,u,F,k] = sparse_rank_one_mex(A,rho,tol,MaxIter,Info,[F],[X0],[k0]) 

// Last Modified: A. d'Aspremont, Laurent El Ghaoui, Ronny Luss, November 2006.
// http://www.carva.org/alexandre.daspremont


#include "SpMaxEig.h" 

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	double *Amat, *Vout, *SqrOut, sig;
	double alpha;
	int n,n2,k,idum=-1,i,j,nsamp,incx=1;
	double *eig,eigbuf,buf;
	double *evec,*bufmata;
	// Arpack params
	char which[2]="LA";
	double tol=1e-6;
	int maxitr=300;
	int info,nummatvec,nconv,arcount,ncv;
	
	
	// Inputs:
	Amat = mxGetPr(prhs[0]);
	//n = mxGetN(prhs[0]);
	n = (int)sqrt(mxGetM(prhs[0])); // Takes input as column vector
	sig = mxGetScalar(prhs[1]);
	nsamp = (int) mxGetScalar(prhs[2]);
	// Outputs:
	plhs[0] = mxCreateDoubleMatrix(n, n, mxREAL);
	Vout = mxGetPr(plhs[0]);
	plhs[1] = mxCreateDoubleMatrix(n, n, mxREAL);
	SqrOut = mxGetPr(plhs[1]);
	plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
	eig = mxGetPr(plhs[2]);
	
	// Allocate
	n2=n*n;
	evec=(double *) calloc(n,sizeof(double));
	bufmata=(double *) calloc(n2,sizeof(double));
	
	// Iterate
	alpha=0.0;cblas_dscal(n,alpha,Vout,incx);cblas_dscal(n,alpha,SqrOut,incx);
	for (k=0;k<nsamp;k++){
		for (i=0;i<n;i++){
			for (j=i;j<n;j++){
				buf=gasdev(&idum);
				bufmata[n*j+i]=Amat[n*j+i]+sig*buf;
				bufmata[n*i+j]=bufmata[n*j+i];
				}
		}
		ncv=4; // TODO: Should get a better size estimate here (cf. arapck manual)
		info=simarpack(bufmata,n,1,ncv,tol,which,maxitr,1,&eigbuf,evec,&nconv,&nummatvec,&arcount);
		if (info!=0) {mexPrintf("SampGrad: Not all eigs converged in Arpack. iter=%d.\n",k);} 
		alpha=1.0/nsamp;
		cblas_dger(CblasColMajor,n,n,alpha,evec,incx,evec,incx,Vout,n);
		for (i=0;i<n;i++){evec[i]=evec[i]*evec[i];}			
		cblas_dger(CblasColMajor,n,n,alpha,evec,incx,evec,incx,SqrOut,n);
		*eig=*eig+eigbuf/nsamp;
	}	
	// Free everything
	free(evec);
	free(bufmata);
}